@@ -1,3 +1,4 @@
+from os import getenv
 import warnings
 import multiprocessing
 from collections import defaultdict
@@ -54,12 +55,25 @@ class ConfiguredSource(BaseModel):
 
 class StatsdConfig(BaseModel):
     host: str = "127.0.0.1"
-    port: int = 8125
+    port: Union[int, str] = 8125
     tags: Dict[str, Union[Loadable, str]] = dict()
     namespace: str = "sovereign"
     enabled: bool = False
     use_ms: bool = True
 
+    @validator("host", pre=True)
+    def load_host(cls, v: str) -> Any:
+        return Loadable.from_legacy_fmt(v).load()
+
+    @validator("port", pre=True)
+    def load_port(cls, v: Union[int, str]) -> Any:
+        if isinstance(v, int):
+            return v
+        elif isinstance(v, str):
+            return Loadable.from_legacy_fmt(v).load()
+        else:
+            raise ValueError(f"Received an invalid port: {v}")
+
     @validator("tags", pre=True)
     def load_tags(cls, v: Dict[str, Union[Loadable, str]]) -> Dict[str, Any]:
         ret = dict()
@@ -73,6 +87,41 @@ class StatsdConfig(BaseModel):
         return ret
 
 
+class DiscoveryCacheConfig(BaseModel):
+    enabled: bool = False
+    host: str = "localhost"
+    port: int = 6379
+    secure: bool = False
+    protocol: str = "redis://"
+    password: SecretStr = SecretStr("")
+    client_side: bool = True  # True = Try in-memory cache before hitting redis
+    wait_for_connection_timeout: int = 5
+    socket_connect_timeout: int = 5
+    socket_timeout: int = 5
+    max_connections: int = 100
+    retry_on_timeout: bool = True  # Retry connections if they timeout.
+    suppress: bool = False  # False = Don't suppress connection errors. True = suppress connection errors
+    socket_keepalive: bool = True  # Try to keep connections to redis around.
+    ttl: int = 60
+
+    @root_validator
+    def set_default_protocol(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        secure = values.get("secure")
+        if secure:
+            values["protocol"] = "rediss://"
+        return values
+
+    @root_validator
+    def set_environmental_variables(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        if host := getenv("SOVEREIGN_DISCOVERY_CACHE_REDIS_HOST"):
+            values["host"] = host
+        if port := getenv("SOVEREIGN_DISCOVERY_CACHE_REDIS_PORT"):
+            values["port"] = int(port)
+        if password := getenv("SOVEREIGN_DISCOVERY_CACHE_REDIS_PASSWORD"):
+            values["password"] = SecretStr(password)
+        return values
+
+
 class XdsTemplate:
     def __init__(self, path: Union[str, Loadable]) -> None:
         if isinstance(path, str):
@@ -91,7 +140,7 @@ class XdsTemplate:
             self.code: Union[Template, ModuleType] = self.loadable.load()
         if isinstance(self.code, ModuleType):
             try:
-                return {"resources": list(self.code.call(*args, **kwargs))}  # type: ignore
+                return {"resources": list(self.code.call(*args, **kwargs))}
             except TypeError as e:
                 if not set(str(e).split()).issuperset(missing_arguments):
                     raise e
@@ -242,6 +291,12 @@ class Resources(List[str]):
         return item in list(self)
 
 
+class Status(BaseModel):
+    code: int
+    message: str
+    details: List[Any]
+
+
 class DiscoveryRequest(BaseModel):
     node: Node = Field(..., title="Node information about the envoy proxy")
     version_info: str = Field(
@@ -257,6 +312,9 @@ class DiscoveryRequest(BaseModel):
     desired_controlplane: str = Field(
         None, title="The host header provided in the Discovery Request"
     )
+    error_detail: Status = Field(
+        None, title="Error details from the previous xDS request"
+    )
 
     @property
     def envoy_version(self) -> str:
@@ -299,10 +357,12 @@ class SovereignAsgiConfig(BaseSettings):
     workers: int = multiprocessing.cpu_count() * 2 + 1
     threads: int = 1
     reuse_port: bool = True
-    preload_app: bool = False
+    preload_app: bool = True
     log_level: str = "warning"
     worker_class: str = "uvicorn.workers.UvicornWorker"
     worker_timeout: int = 30
+    worker_tmp_dir: str = "/dev/shm"
+    graceful_timeout: int = worker_timeout * 2
 
     class Config:
         fields = {
@@ -326,6 +386,8 @@ class SovereignAsgiConfig(BaseSettings):
             "threads": self.threads,
             "workers": self.workers,
             "worker_class": self.worker_class,
+            "worker_tmp_dir": self.worker_tmp_dir,
+            "graceful_timeout": self.graceful_timeout,
         }
 
 
@@ -357,6 +419,7 @@ class SovereignConfig(BaseSettings):
     enable_access_logs: bool = True
     log_fmt: Optional[str] = ""
     ignore_empty_log_fields: bool = False
+    discovery_cache: DiscoveryCacheConfig = DiscoveryCacheConfig()
 
     class Config:
         fields = {
@@ -616,6 +679,7 @@ class SovereignConfigv2(BaseSettings):
     sentry_dsn: SecretStr = SecretStr("")
     debug: bool = False
     legacy_fields: LegacyConfig = LegacyConfig()
+    discovery_cache: DiscoveryCacheConfig = DiscoveryCacheConfig()
 
     class Config:
         fields = {
@@ -712,4 +776,5 @@ class SovereignConfigv2(BaseSettings):
                 dns_hard_fail=other.dns_hard_fail,
                 environment=other.environment,
             ),
+            discovery_cache=other.discovery_cache,
         )
