@@ -4,14 +4,33 @@ from fastapi import Body, Header
 from fastapi.routing import APIRouter
 from fastapi.responses import Response
 
-from sovereign import discovery, logs
+from sovereign import discovery, logs, config
 from sovereign.utils.auth import authenticate
+from sovereign.utils.version_info import compute_hash
 from sovereign.schemas import (
     DiscoveryRequest,
     DiscoveryResponse,
     ProcessedTemplate,
 )
 
+discovery_cache = config.discovery_cache
+
+if discovery_cache.enabled:
+    from cashews import cache
+
+    cache.setup(
+        f"{discovery_cache.protocol}{discovery_cache.host}:{discovery_cache.port}",
+        password=discovery_cache.password.get_secret_value(),
+        client_side=discovery_cache.client_side,
+        wait_for_connection_timeout=discovery_cache.wait_for_connection_timeout,
+        socket_connect_timeout=discovery_cache.socket_connect_timeout,
+        socket_timeout=discovery_cache.socket_timeout,
+        max_connections=discovery_cache.max_connections,
+        retry_on_timeout=discovery_cache.retry_on_timeout,
+        suppress=discovery_cache.suppress,
+        socket_keepalive=discovery_cache.socket_keepalive,
+        enable=discovery_cache.enabled,
+    )
 
 router = APIRouter()
 
@@ -66,13 +85,17 @@ async def discovery_response(
     host: str = Header("no_host_provided"),
 ) -> Response:
     discovery_request.desired_controlplane = host
-    response = perform_discovery(discovery_request, version, xds_type, skip_auth=False)
+    response = await perform_discovery(
+        discovery_request, version, xds_type, skip_auth=False
+    )
     logs.queue_log_fields(
         XDS_RESOURCES=discovery_request.resource_names,
         XDS_ENVOY_VERSION=discovery_request.envoy_version,
         XDS_CLIENT_VERSION=discovery_request.version_info,
         XDS_SERVER_VERSION=response.version,
     )
+    if discovery_request.error_detail:
+        logs.queue_log_fields(XDS_ERROR_DETAIL=discovery_request.error_detail.message)
     headers = response_headers(discovery_request, response, xds_type)
 
     if response.version == discovery_request.version_info:
@@ -86,7 +109,7 @@ async def discovery_response(
     return Response(content="Resources could not be determined", status_code=500)
 
 
-def perform_discovery(
+async def perform_discovery(
     req: DiscoveryRequest,
     api_version: str,
     resource_type: str,
@@ -94,13 +117,39 @@ def perform_discovery(
 ) -> ProcessedTemplate:
     if not skip_auth:
         authenticate(req)
+    if discovery_cache.enabled:
+        logs.queue_log_fields(CACHE_XDS_HIT=False)
+        cache_key = compute_hash(
+            [
+                api_version,
+                resource_type,
+                req.envoy_version,
+                req.resource_names,
+                req.desired_controlplane,
+                req.hide_private_keys,
+                req.type_url,
+                req.node.cluster,
+                req.node.locality,
+                req.node.metadata.get("auth", None),
+                req.node.metadata.get("num_cpus", None),
+            ]
+        )
+        if template := await cache.get(key=cache_key, default=None):
+            logs.queue_log_fields(CACHE_XDS_HIT=True)
+            return template  # type: ignore[no-any-return]
     template = discovery.response(req, resource_type)
     type_url = type_urls.get(api_version, {}).get(resource_type)
     if type_url is not None:
         for resource in template.resources:
             if not resource.get("@type"):
                 resource["@type"] = type_url
-    return template
+    if discovery_cache.enabled:
+        await cache.set(
+            key=cache_key,
+            value=template,
+            expire=discovery_cache.ttl,
+        )
+    return template  # type: ignore[no-any-return]
 
 
 def not_modified(headers: Dict[str, str]) -> Response:
